#!/usr/bin/python

"""
game.py - main in-game scene classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources
import tilemap
import player

from common import Camera, FadeInOut, MenuButton, MenuList

class ProgressData(object):
    def __init__(self):
        self.current_level = "level_computerroom"
        self.got_blue_scroll = False
        self.got_green_scroll = False
        self.got_red_scroll = False
    
    def LoadProgressData(self):
        f = open(self.savepath, "r");
        self.current_level = f.readline().split()[0]
        self.got_blue_scroll = bool(int(f.readline().split()[0]))
        self.got_green_scroll = bool(int(f.readline().split()[0]))
        self.got_red_scroll = bool(int(f.readline().split()[0]))
        f.close()
    
    def SaveProgressData(self):
        f = open(self.savepath, "w");
        f.write('%s\n'%(self.current_level))
        f.write('%d\n'%(int(self.got_blue_scroll)))
        f.write('%d\n'%(int(self.got_green_scroll)))
        f.write('%d\n'%(int(self.got_red_scroll)))
        f.close()
    
    def Reset(self):
        self.current_level = "level_computerroom"
        self.got_blue_scroll = False
        self.got_green_scroll = False
        self.got_red_scroll = False

class MainGame(GameScene):
    def __init__(self, director, window_size):
        super(MainGame, self).__init__(director)
        self.window_size = window_size
        
        # frame rate recording
        self.avgframerate = -1
        self.frsamples = 0
        
        # fade in/out
        self.fade = FadeInOut(30)
        
        # Music
        self.current_music = 'none'
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        # Initialise player data
        self.progress_data = ProgressData()
    
    def on_switchto(self, switchtoargs):
        
        lvlreset = switchtoargs[0]
        if not lvlreset:
            return
        
        level_id = switchtoargs[1]
        ingress_id = switchtoargs[2]
        
        # Save progress
        self.progress_data.current_level = level_id
        self.progress_data.SaveProgressData()
        
        # Check for end-game trigger
        if self.progress_data.got_blue_scroll and self.progress_data.got_green_scroll and self.progress_data.got_red_scroll:
            self.director.change_scene('wingamescene', resources.wingamescene_data)
        
        # Initialise objects
        self.camera = Camera(self.window_size,stickyness=0.1)
        self.tiledlayers = tilemap.TiledLayers()
        self.player = player.Player()
        
        # set cross handles
        self.tiledlayers.set_handles((self.camera, self.player, self))
        self.player.set_handles((self.camera, self.tiledlayers))
        
        # load level data
        self.tiledlayers.load_level(level_id, ingress_id)
        
        # Check music
        if not self.current_music == resources.initleveldata[level_id].music:
            self.current_music = resources.initleveldata[level_id].music
            pygame.mixer.music.stop()
            if not self.current_music == 'none':
                pygame.mixer.music.load(resources.musicpaths[self.current_music])
                pygame.mixer.music.set_volume(0.5)
                pygame.mixer.music.play(-1)
                
        # Fade in game
        self.background.fill((0,0,0))
        self.fade.FadeIn()
        
    def on_update(self):
        
        # framerate tracking
        self.frsamples += 1
        if self.frsamples == 1:
            self.avgframerate = self.director.framerate
        else:
            self.avgframerate = self.avgframerate + (self.director.framerate - self.avgframerate)/(self.frsamples)
        
        # Update objects
        self.player.UpdateMotion()
        self.camera.UpdateCamera([self.player.x,self.player.y])
        self.tiledlayers.Update()
        
        # Control fade in/out, look for end game cues
        self.fade.Update()
        if self.player.exiting and self.fade.direction == 'in':
            if self.player.destination[0] in ["scroll_red","scroll_green","scroll_blue"]:
                musicfade = True
            else:
                if not self.current_music == resources.initleveldata[self.player.destination[0]].music:
                    musicfade = True
                else:
                    musicfade = False
            self.fade.FadeOut(musicfade)
        if self.player.dying and self.fade.direction == 'in':
            self.background.fill((255,0,0))
            self.fade.FadeOut()
        if self.fade.finished_out:
            if self.player.dying:
                self.director.change_scene('death', [self.tiledlayers.level_id])
            else:
                if self.player.destination[0] in ["scroll_red","scroll_green","scroll_blue"]:
                    self.current_music = 'none'
                    pygame.mixer.music.stop()
                    if self.player.destination[0] == "scroll_red":
                        self.progress_data.got_red_scroll = True
                    elif self.player.destination[0] == "scroll_green":
                        self.progress_data.got_green_scroll = True
                    elif self.player.destination[0] == "scroll_blue":
                        self.progress_data.got_blue_scroll = True
                    self.director.change_scene('gotscroll', [self.player.destination[0]])
                else:
                    self.director.change_scene('maingame', [True,self.player.destination[0],self.player.destination[1]])
        
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.draw(self.h_pausescene.background)
                self.director.change_scene('pausescene', [self.tiledlayers.level_id])
            if event.type == KEYDOWN or event.type == KEYUP:
                if event.type == KEYDOWN and event.key == K_r: # instant reset
                    self.director.change_scene('maingame', [True,self.tiledlayers.level_id,self.tiledlayers.ingress_id])
                self.player.control.ProcessKeyEvent(event)
    
    def draw(self, screen):
        screen.blit(self.tiledlayers.bglayer_bgcolour, (0, 0))
        self.tiledlayers.RenderMidLayer(screen)
        self.tiledlayers.batgroup.sprite_group.draw(screen)
        self.tiledlayers.buttoncontrol.DrawButtons(screen)
        self.tiledlayers.movingblocks_spritegroup.draw(screen)
        self.tiledlayers.hurtblockcontrol.DrawHurtBlocks(screen)
        self.tiledlayers.doorcontrol.DrawDoors(screen)
        self.player.Draw(screen)
        self.tiledlayers.platformcontrol.DrawPlatforms(screen)
        self.tiledlayers.RenderFGLayer(screen)
    
    def on_draw(self, screen):
        
        self.draw(screen)
        self.background.set_alpha(self.fade.alpha)
        screen.blit(self.background, (0, 0))

class Death(GameScene):
    def __init__(self, director, window_size):
        super(Death, self).__init__(director)
        self.window_size = window_size
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((255,0,0))
        self.background.convert()
        
        self.fadebackground = pygame.Surface(window_size)
        self.fadebackground.fill((0,0,0))
        self.fadebackground.convert()
        
        self.fade = FadeInOut(30)
        self.exiting = False
        
    def on_switchto(self, switchtoargs):
        self.level_id = switchtoargs[0]
        self.exiting = False
        self.fade.alpha = 0
        self.fade.finished_in = True
        self.fade.finished_out = False
        self.fade.direction = 'in'
        
    def on_update(self):
        
        # Check for return to game
        self.fade.Update()
        if self.fade.finished_in and self.fade.direction == 'in':
            self.fade.FadeOut()
        if self.fade.finished_out:
            self.director.change_scene('maingame', [True,self.level_id,-1])
        
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.director.change_scene(None, [])
                
    def on_draw(self, screen):
        screen.blit(self.background, (0, 0))
        self.fadebackground.set_alpha(self.fade.alpha)
        screen.blit(self.fadebackground, (0, 0))

class PauseScreen(GameScene):
    def __init__(self, director, window_size):
        super(PauseScreen, self).__init__(director)
        self.window_size = window_size
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
    
    def resume_game_now(self):
        self.director.change_scene('maingame', [False,-1,-1])
    
    def reset_game_now(self):
        self.director.change_scene('maingame', [True,self.from_level,-1])
    
    def quit_game_now(self):
        self.director.change_scene('titlescene', [])
    
    def on_switchto(self, switchtoargs):
        
        self.from_level = switchtoargs[0]
        
        # Setup background
        backfade = pygame.Surface(self.background.get_size())
        backfade.fill((0,0,0))
        backfade.convert()
        backfade.set_alpha(128)
        self.background.blit(backfade, (0,0))
        
        # top-level menu buttons
        self.pos_h = (self.window_size[0]/2)-((resources.pausescreen_surfs['bg'].get_width()+resources.pausescreen_surfs['help'].get_width()+40)/2)
        self.pos_h2 = self.pos_h + resources.pausescreen_surfs['bg'].get_width() + 40
        self.pos_v = (self.window_size[1]/2)-(resources.pausescreen_surfs['bg'].get_height()/2)
        self.pos_v2 = (self.window_size[1]/2)-(resources.pausescreen_surfs['help'].get_height()/2)
        self.button_resume = MenuButton((self.pos_h,self.pos_v+26),resources.pausescreen_surfs['resume_on'],resources.pausescreen_surfs['resume_off'],self.resume_game_now)
        self.button_reset = MenuButton((self.pos_h,self.pos_v+40),resources.pausescreen_surfs['reset_on'],resources.pausescreen_surfs['reset_off'],self.reset_game_now)
        self.button_quit = MenuButton((self.pos_h,self.pos_v+54),resources.pausescreen_surfs['quit_on'],resources.pausescreen_surfs['quit_off'],self.quit_game_now)
        self.buttonlist = [self.button_resume,self.button_reset,self.button_quit]
        self.topmenus = MenuList(self.buttonlist,0,resources.controlmap)
        
        # reset menu
        self.topmenus.select_ind = 0
        
    def on_update(self):
        pass
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.director.change_scene('maingame', [False,-1,-1])
            if event.type == KEYDOWN:
                self.topmenus.ProcessKeyEvent(event)
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        screen.blit(resources.pausescreen_surfs['bg'], (self.pos_h,self.pos_v))
        screen.blit(resources.pausescreen_surfs['help'], (self.pos_h2,self.pos_v2))
        self.topmenus.Draw(screen)
