#!/usr/bin/python

"""
tilemap.py - tilemap classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *

import resources
import player
import killerbat

class BlockSprite(pygame.sprite.DirtySprite):
    def __init__(self,size):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = resources.tiles
        self.rect = self.image.get_rect()
        self.source_rect = pygame.Rect((0,0),(size[0],size[1]))
        self.dirty = 2

class SwitchButton(object):
    def __init__(self,tile,type,h_mechs,id):
        self.id = id
        self.tile = tile
        self.type = type
        self.h_mechs = h_mechs
        self.state = False
    
    def Update(self,trigger):
        if trigger:
            self.state = True
            for h in self.h_mechs:
                h.state = True
            #print self.id, "on"
        elif self.type == 'mom':
            self.state = False
            for h in self.h_mechs:
                h.state = False

class ButtonControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.buttons = []
    
    def AddButton(self,tile,type,h_mechs):
        id = len(self.buttons)+200
        self.buttons.append(SwitchButton(tile,type,h_mechs,id))
        self.parent.InsertObj(tile,id)
    
    def UpdateButtons(self):
        ts = self.parent.tilesize
        for b in self.buttons:
            steptile = b.tile - self.parent.map_size[0]
            steptiles = [steptile-1,steptile,steptile+1]
            stepped_on = False
            prev_state = b.state
            for st in steptiles:
                for id in self.parent.objectlayer[st]:
                    if id == 0 and self.parent.h_player.ground_state:
                        condx1 = self.parent.h_player.x+(self.parent.h_player.sx/2) > ts*(b.tile%self.parent.map_size[0])
                        condx2 = self.parent.h_player.x-(self.parent.h_player.sx/2) < ts*(b.tile%self.parent.map_size[0])+ts
                        if condx1 and condx2:
                            stepped_on = True
                    elif id >= 100 and id < 200:
                        block = self.parent.movingblocks[id-100]
                        if block.ground_state:
                            condx1 = (block.x+ts/2) > (ts*(b.tile%self.parent.map_size[0]))
                            condx2 = (block.x-ts/2) < (ts*(b.tile%self.parent.map_size[0])+ts)
                            if condx1 and condx2:
                                stepped_on = True
            if stepped_on:
                b.Update(True)
            else:
                b.Update(False)
            if not prev_state == b.state:
                resources.soundfx['button'].play()
    
    def DrawButtons(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        for b in self.buttons:
            coords = [(b.tile%self.parent.map_size[0])*self.parent.tilesize, (b.tile/self.parent.map_size[0])*self.parent.tilesize]
            if b.state:
                tilecoords = resources.tiles_coords_buttons[b.type][1]
            else:
                tilecoords = resources.tiles_coords_buttons[b.type][0]
                coords[1] = coords[1]-4
            screen.blit(resources.tiles, (coords[0]-camerax,coords[1]-cameray), area=tilecoords)

class HurtBlock(object):
    def __init__(self,tile,type,id):
        self.tile = tile
        self.type = type
        self.id = id
        self.state = False
        self.switched_on = True
        self.frame = 0
        self.tick = 0
        if self.type == 'lava':
            self.frame_to = 6
            self.nframes = 6
        else:
            self.frame_to = 3
            self.nframes = 5
    
    def Update(self):
        self.tick += 1
        if (self.tick%self.frame_to) == 0:
            self.frame = (self.frame+1)%self.nframes
        if self.state:
            if self.type == 'normal':
                self.switched_on = False
            elif self.type == 'inverse':
                self.switched_on = True
        else:
            if self.type == 'normal':
                self.switched_on = True
            elif self.type == 'inverse':
                self.switched_on = False

class HurtBlockControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.blocks = []
        
    def AddHurtBlock(self,tile,type):
        id = len(self.blocks)+400
        self.blocks.append(HurtBlock(tile,type,id))
        self.parent.InsertObj(tile,id)
    
    def UpdateHurtBlocks(self):
        for block in self.blocks:
            block.Update()
    
    def DrawHurtBlocks(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        for block in self.blocks:
            x = self.parent.tilesize*(block.tile%self.parent.map_size[0])
            y = self.parent.tilesize*(block.tile/self.parent.map_size[0])
            if block.type == 'lava':
                tilecoords = resources.tiles_coords[resources.tiles_hurtblockseq['lava'][block.frame]]
            else:
                base_coord = self.parent.tilelayer_mg[block.tile]
                if block.switched_on:
                    tilecoords = resources.tiles_coords[base_coord+resources.tiles_hurtblockseq['elecadd'][block.frame]]
                else:
                    tilecoords = resources.tiles_coords[base_coord]
            screen.blit(resources.tiles, (x-camerax,y-cameray), area=tilecoords)

class MovingPlatform(object):
    def __init__(self,x,miny,maxy,attach_y,speed,type,id):
        self.miny = miny
        self.maxy = maxy
        self.attach_y = attach_y
        self.speed = speed
        self.type = type
        self.id = id
        
        self.x = x
        if self.type == 'raise':
            self.y = self.maxy
        elif type == 'lower':
            self.y = self.miny
        
        self.state = False
    
    def Update(self):
        if self.state:
            if self.type == 'raise':
                self.y = max(self.y-self.speed,self.miny)
            elif self.type == 'lower':
                self.y = min(self.y+self.speed,self.maxy)
        else:
            if self.type == 'lower':
                self.y = max(self.y-self.speed,self.miny)
            elif self.type == 'raise':
                self.y = min(self.y+self.speed,self.maxy)

class MovingPlatformControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.platforms = []
        
    def AddPlatform(self,tile_top,tile_bottom,attach_tile,type,speed):
        ts = self.parent.tilesize
        id = len(self.platforms)+300
        x = self.parent.tilesize*(tile_top%self.parent.map_size[0])
        maxy = ts*(tile_bottom/self.parent.map_size[0])+(ts/2)
        miny = ts*(tile_top/self.parent.map_size[0])+(ts/2)
        attach_y = self.parent.tilesize*(attach_tile/self.parent.map_size[0])
        self.platforms.append(MovingPlatform(x,miny,maxy,attach_y,speed,type,id))
        if type == 'lower':
            self.parent.InsertObj(tile_top,id)
            self.parent.InsertObj(tile_top+1,id)
            self.parent.InsertObj(tile_top+2,id)
        elif type == 'raise':
            self.parent.InsertObj(tile_bottom,id)
            self.parent.InsertObj(tile_bottom+1,id)
            self.parent.InsertObj(tile_bottom+2,id)
    
    def UpdatePlatforms(self):
        ts = self.parent.tilesize
        for p in self.platforms:
            init_tile = self.parent.map_size[0]*(p.y/ts)+((p.x+(ts/2))/ts)
            p.Update()
            final_tile = self.parent.map_size[0]*(p.y/ts)+((p.x+(ts/2))/ts)
            if not init_tile == final_tile:
                self.parent.UpdateObj(init_tile,final_tile,p.id)
                self.parent.UpdateObj(init_tile+1,final_tile+1,p.id)
                self.parent.UpdateObj(init_tile+2,final_tile+2,p.id)
    
    def DrawPlatforms(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        for p in self.platforms:
            tilecoords1 = resources.tiles_coords[resources.tiles_platform['base_coords'][0]]
            tilecoords2 = resources.tiles_coords[resources.tiles_platform['base_coords'][1]]
            tilecoords3 = resources.tiles_coords[resources.tiles_platform['base_coords'][2]]
            screen.blit(resources.tiles, (p.x-camerax,p.y-(ts/2)-cameray), area=tilecoords1)
            screen.blit(resources.tiles, (p.x+ts-camerax,p.y-(ts/2)-cameray), area=tilecoords2)
            screen.blit(resources.tiles, (p.x+2*ts-camerax,p.y-(ts/2)-cameray), area=tilecoords3)
            
            link_len = 12
            linkbase_len = 14
            tilecoords_chain_left = resources.tiles_platform['chainlink']['left']
            tilecoords_chain_left_partial = list(tilecoords_chain_left)
            tilecoords_chain_right = resources.tiles_platform['chainlink']['right']
            tilecoords_chain_right_partial = list(tilecoords_chain_right)
            chain_len = p.y-(ts/2)-p.attach_y-linkbase_len
            nlinks = chain_len/link_len
            offset = chain_len%link_len
            tilecoords_chain_left_partial[1] = tilecoords_chain_left_partial[1] + (link_len-offset)
            tilecoords_chain_left_partial[3] = offset
            tilecoords_chain_right_partial[1] = tilecoords_chain_right_partial[1] + (link_len-offset)
            tilecoords_chain_right_partial[3] = offset
            
            screen.blit(resources.tiles, (p.x-camerax,p.y-(ts/2)-ts-cameray), area=resources.tiles_platform['chainbase']['left'])
            screen.blit(resources.tiles, (p.x+2*ts-camerax,p.y-(ts/2)-ts-cameray), area=resources.tiles_platform['chainbase']['right'])
            for i in xrange(nlinks):
                screen.blit(resources.tiles, (p.x-camerax,p.y-(ts/2)-linkbase_len-link_len*(i+1)-cameray), area=tilecoords_chain_left)
                screen.blit(resources.tiles, (p.x+2*ts-camerax,p.y-(ts/2)-linkbase_len-link_len*(i+1)-cameray), area=tilecoords_chain_right)
            screen.blit(resources.tiles, (p.x-camerax,p.attach_y-cameray), area=tilecoords_chain_left_partial)
            screen.blit(resources.tiles, (p.x+2*ts-camerax,p.attach_y-cameray), area=tilecoords_chain_right_partial)

class Door(object):
    def __init__(self,tile,type,opentype,id,h_tiledlayers):
        self.tile = tile
        self.type = type
        self.opentype = opentype
        self.id = id
        self.h_tiledlayers = h_tiledlayers
        
        self.state = False
        self.nframes = 4
        self.ticksperframe = 3
        self.closedpos = self.nframes*self.ticksperframe
        if self.opentype == 'closed':
            self.position = self.closedpos # fully closed
        elif self.opentype == 'open':
            self.position = 0 # fully open
    
    def SetDoorOcc(self,direction):
        mapsizex = self.h_tiledlayers.map_size[0]
        occlayer = self.h_tiledlayers.occlayer
        if direction == 'open':
            occval = 0
        elif direction == 'closed':
            occval = 1
        if self.type == 'vertical':
            occlayer[self.tile+mapsizex] = occval
            occlayer[self.tile+2*mapsizex] = occval
        elif self.type == 'horizontal':
            occlayer[self.tile+1] = occval
            occlayer[self.tile+2] = occval
    
    def Update(self):
        #print self.state
        if self.opentype == 'closed' and not self.state: # closed door, switched off
            self.position = min(self.position+1,self.closedpos)
        elif self.opentype == 'closed' and self.state: # closed door, switched on
            self.position = max(self.position-1,0)
        elif self.opentype == 'open' and not self.state: # open door, switched off
            self.position = max(self.position-1,0)
        elif self.opentype == 'open' and self.state: # open door, switched on
            self.position = min(self.position+1,self.closedpos)
        if self.position == 0:
            self.SetDoorOcc('open')
        else:
            self.SetDoorOcc('closed')

class DoorControl(object):
    def __init__(self,parent):
        self.parent = parent
        self.doors = []
        
    def AddDoor(self,tile,type,opentype):
        id = len(self.doors)+500
        self.doors.append(Door(tile,type,opentype,id,self.parent))
    
    def UpdateDoors(self):
        for d in self.doors:
            d.Update()
    
    def DrawDoors(self,screen):
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        ts = self.parent.tilesize
        msx = self.parent.map_size[0]
        for d in self.doors:
            if d.type == 'vertical':
                tiles = [d.tile,d.tile+msx,d.tile+2*msx]
            elif d.type == 'horizontal':
                tiles = [d.tile,d.tile+1,d.tile+2]
            x1 = ts*(tiles[0]%msx)
            y1 = ts*(tiles[0]/msx)
            x2 = ts*(tiles[1]%msx)
            y2 = ts*(tiles[1]/msx)
            x3 = ts*(tiles[2]%msx)
            y3 = ts*(tiles[2]/msx)
            frame = d.position/d.ticksperframe
            tilecoords1 = resources.tiles_coords[resources.tiles_doorseq[d.type][0][frame]]
            tilecoords2 = resources.tiles_coords[resources.tiles_doorseq[d.type][1][frame]]
            tilecoords3 = resources.tiles_coords[resources.tiles_doorseq[d.type][2][frame]]
            screen.blit(resources.tiles, (x1-camerax,y1-cameray), area=tilecoords1)
            screen.blit(resources.tiles, (x2-camerax,y2-cameray), area=tilecoords2)
            screen.blit(resources.tiles, (x3-camerax,y3-cameray), area=tilecoords3)
            

class MovingBlock(object):
    def __init__(self,parent,init_tile,id):
        self.parent = parent
        self.x = self.parent.tilesize*(init_tile%self.parent.map_size[0])+(self.parent.tilesize/2)
        self.y = self.parent.tilesize*(init_tile/self.parent.map_size[0])+(self.parent.tilesize/2)-1
        self.vy = 0
        self.vx = 0
        self.ground_state = True
        self.id = id
        self.parent.InsertObj(init_tile,self.id)
        
        self.sprite = BlockSprite((self.parent.tilesize,self.parent.tilesize))
    
    def UpdateMotion(self):
        
        init_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        
        if self.ground_state == False:
            self.vy = min(self.vy + 1, 5)
        else:
            self.vy = 0
        self.y = self.y + self.vy
        self.x = self.x + self.vx
        
        (self.x,self.y) = self.parent.HandleObjectWallCollision((self.x,self.y),(self.parent.tilesize,self.parent.tilesize),(0,self.vy))
        
        # check for on top of other boxes
        current_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        idlist = self.parent.FindMechanisms(current_tile,'block')
        ts = self.parent.tilesize
        ground_state_on_block = False
        for id in idlist:
            if id == self.id:
                continue
            block = self.parent.movingblocks[id-100]
            checkx2 = self.x+(ts/2) > (block.x-ts/2) and self.x-(ts/2) < (block.x+ts/2)
            checky2 = self.y+(ts/2) >= (block.y-ts/2) and self.y+(ts/2) < (block.y-(ts/2)+15)
            if checkx2 and checky2 and self.vy >= 0:
                ground_state_on_block = True
                self.y = block.y-ts
        
        # check for on top of platforms
        current_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        idlist = self.parent.FindMechanisms(current_tile,'platform')
        ts = self.parent.tilesize
        ground_state_on_platform = False
        for id in idlist:
            platform = self.parent.platformcontrol.platforms[id-300]
            checkx2 = self.x+(ts/2) > (platform.x-ts/2) and self.x-(ts/2) < (platform.x+ts*2+ts/2)
            checky2 = self.y+(ts/2) >= (platform.y-ts/2-1) and self.y+(ts/2) < (platform.y-(ts/2)+15)
            if checkx2 and checky2:
                ground_state_on_platform = True
                self.y = platform.y-ts-1
        
        # Check if need to inform tilemap object layer of updates
        final_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        if not init_tile == final_tile:
            self.parent.UpdateObj(init_tile,final_tile,self.id)
        
        # Check for ground state
        leftfoottile = self.parent.map_size[0]*((self.y+(self.parent.tilesize/2)+1)/self.parent.tilesize)+((self.x-(self.parent.tilesize/2))/self.parent.tilesize)
        rightfoottile = self.parent.map_size[0]*((self.y+(self.parent.tilesize/2)+1)/self.parent.tilesize)+((self.x+(self.parent.tilesize/2))/self.parent.tilesize)
        if self.parent.occlayer[leftfoottile] == 1 or self.parent.occlayer[rightfoottile] == 1 or ground_state_on_block or ground_state_on_platform:
            self.ground_state = True
            self.vx = 0
        else:
            self.ground_state = False
    
    def MoveBlock(self, objmovevec):
        init_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        vx = objmovevec[0]
        vy = objmovevec[1]
        (self.x,self.y) = self.parent.HandleObjectWallCollision((self.x+vx,self.y+vy),(self.parent.tilesize,self.parent.tilesize),(vx,vy))
        final_tile = self.parent.map_size[0]*(self.y/self.parent.tilesize)+(self.x/self.parent.tilesize)
        if not init_tile == final_tile:
            self.parent.UpdateObj(init_tile,final_tile,self.id)
        
        idlist = self.parent.FindMechanisms(final_tile,'block')
        ts = self.parent.tilesize
        for id in idlist:
            if id == self.id:
                continue
            block = self.parent.movingblocks[id-100]
            checkx1 = self.x+(ts/2) > (block.x-ts/2) and self.x+(ts/2) < (block.x+ts/2) and vx > 0
            checkx2 = self.x-(ts/2) < (block.x+ts/2) and self.x-(ts/2) > (block.x-ts/2) and vx < 0
            checky = self.y+(ts/2) > (block.y-ts/2) and self.y-(ts/2) < (block.y+ts/2)
            if checkx1 and checky:
                self.x = block.x - ts - 1
            elif checkx2 and checky:
                self.x = block.x + ts + 1
    
    def UpdateSprite(self):
        src_coords = resources.tiles_coords[19]
        self.sprite.source_rect.x = src_coords[0]
        self.sprite.source_rect.y = src_coords[1]
        (camerax,cameray) = (self.parent.h_camera.x-self.parent.h_camera.w_view/2,self.parent.h_camera.y-self.parent.h_camera.h_view/2)
        self.sprite.rect.x = int(self.x-(self.parent.tilesize/2)-camerax)
        self.sprite.rect.y = int(self.y-(self.parent.tilesize/2)-cameray)

class TiledLayers(object):
    
    def __init__(self):
        pass
    
    def set_handles(self,handles):
        (self.h_camera, self.h_player, self.parent) = handles
    
    def load_level(self, level_id, ingress_id):
        self.level_id = level_id
        self.ingress_id = ingress_id
        self.map_size = resources.initleveldata[level_id].map_size[:]
        self.bgcolour = resources.initleveldata[level_id].bgcolour[:]
        self.tilelayer_bg = resources.initleveldata[level_id].tilelayer_bg[:]
        self.tilelayer_mg = resources.initleveldata[level_id].tilelayer_mg[:]
        self.tilelayer_fg = resources.initleveldata[level_id].tilelayer_fg[:]
        self.occlayer = resources.initleveldata[level_id].occlayer[:]
        self.objectlayer = [[] for i in xrange(self.map_size[0]*self.map_size[1])]
        
        # Hack to colour scrolls on wall in level_computerroom
        if self.level_id == "level_computerroom":
            if self.parent.progress_data.got_blue_scroll:
                self.tilelayer_mg[186] = 96
                self.tilelayer_mg[187] = 97
                self.tilelayer_mg[216] = 112
                self.tilelayer_mg[217] = 113
            if self.parent.progress_data.got_green_scroll:
                self.tilelayer_mg[188] = 100
                self.tilelayer_mg[189] = 101
                self.tilelayer_mg[218] = 116
                self.tilelayer_mg[219] = 117
            if self.parent.progress_data.got_red_scroll:
                self.tilelayer_mg[190] = 98
                self.tilelayer_mg[191] = 99
                self.tilelayer_mg[220] = 114
                self.tilelayer_mg[221] = 115
        
        # Object layer reference:
        #
        # 0: player
        # 100-199: moveable boxes
        # 200-299: buttons
        # 300-399: platforms
        # 400-499: hurtblock
        # 500-599: doors
        # 1000-1999: killer bats
        
        self.tilesize = 16
        
        # send initial data to camera
        self.h_camera.xlim = self.map_size[0]*self.tilesize
        self.h_camera.ylim = self.map_size[1]*self.tilesize
        
        # Add level exit
        self.exit_data = resources.initleveldata[level_id].exit_data[:]
        
        # Initialise player into object layer
        if ingress_id == -1:
            player_start_tile = resources.initleveldata[level_id].player_start_tile
        else:
            player_start_tile = self.exit_data[ingress_id][0]
        self.h_player.x = self.tilesize*(player_start_tile%self.map_size[0])+8
        self.h_player.y = self.tilesize*(player_start_tile/self.map_size[0])-1
        self.InsertObj(player_start_tile,0)
        
        # Initialise moving blocks into map
        self.movingblocks = []
        self.movingblocks_spritegroup = pygame.sprite.LayeredDirty([])
        for i in xrange(resources.initleveldata[level_id].movingblockdata[0]):
            tile = resources.initleveldata[level_id].movingblockdata[i+1]
            id = 100+i
            self.movingblocks.append(MovingBlock(self,tile,id=id))
            self.movingblocks_spritegroup.add(self.movingblocks[-1].sprite)
        
        # Initialise hurtblocks into map
        self.hurtblockcontrol = HurtBlockControl(self)
        for block in resources.initleveldata[level_id].hurtblockdata:
            tile = block[0]
            type = block[1]
            self.hurtblockcontrol.AddHurtBlock(tile,type)
        
        # Initialise Moving Platforms
        self.platformcontrol = MovingPlatformControl(self)
        for platform in resources.initleveldata[level_id].platformdata:
            tile_top = platform[0]
            tile_bottom = platform[1]
            attach_tile = platform[2]
            type = platform[3]
            speed = platform[4]
            self.platformcontrol.AddPlatform(tile_top,tile_bottom,attach_tile,type,speed)
        
        # Initialise doors
        self.doorcontrol = DoorControl(self)
        for door in resources.initleveldata[level_id].doordata:
            tile = door[0]
            type = door[1]
            opentype = door[2]
            self.doorcontrol.AddDoor(tile,type,opentype)
        
        # Initialise Switch Buttons into map
        self.buttoncontrol = ButtonControl(self)
        for button in resources.initleveldata[level_id].buttondata:
            tile = button[0]
            type = button[1]
            mechs = button[2]
            hmechs = []
            for m in mechs:
                typem = m[0]
                ind = m[1]
                if typem == 'platform':
                    hmechs.append(self.platformcontrol.platforms[ind])
                elif typem == 'hurtblock':
                    hmechs.append(self.hurtblockcontrol.blocks[ind])
                elif typem == 'door':
                    hmechs.append(self.doorcontrol.doors[ind])
            self.buttoncontrol.AddButton(tile,type,hmechs)
        
        # Initialise bats
        self.batgroup = killerbat.BatGroup(self,self.h_camera,self.h_player)
        for i in xrange(resources.initleveldata[level_id].killerbatdata[0]):
            tile = resources.initleveldata[level_id].killerbatdata[i+1]
            self.batgroup.AddBat(tile)
        
        # background colour layer
        self.bglayer_bgcolour = pygame.Surface((16*self.map_size[0],16*self.map_size[1]))
        self.bglayer_bgcolour.fill(self.bgcolour)
        self.bglayer_bgcolour.convert()
        
        # render tiled layers
        self.bglayer_t = pygame.Surface((16*self.map_size[0],16*self.map_size[1]))
        self.bglayer_t.fill((255,0,255))
        self.bglayer_t.convert()
        self.bglayer_t.set_colorkey((255,0,255))
        self.bglayer_tfg = pygame.Surface((16*self.map_size[0],16*self.map_size[1]))
        self.bglayer_tfg.fill((255,0,255))
        self.bglayer_tfg.convert()
        self.bglayer_tfg.set_colorkey((255,0,255))
        for i in range(self.map_size[0]):
            for j in range(self.map_size[1]):
                coords = [i*self.tilesize, j*self.tilesize]
                tilevalbg = self.tilelayer_bg[self.map_size[0]*j+i]
                tilevalmg = self.tilelayer_mg[self.map_size[0]*j+i]
                tilevalfg = self.tilelayer_fg[self.map_size[0]*j+i]
                if tilevalbg >= 0:
                    tilecoords = resources.tiles_coords[tilevalbg]
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                if tilevalmg >= 0:
                    tilecoords = resources.tiles_coords[tilevalmg]
                    """
                    if not tileval in [286,294,308,316,330,338]:
                        self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                    """
                    self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
                if tilevalfg >= 0:
                    tilecoords = resources.tiles_coords[tilevalfg]
                    self.bglayer_tfg.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
        
    def UpdateTileLayer(self, tile, tileval):
        coords = [(tile%self.map_size[0])*self.tilesize, (tile/self.map_size[0])*self.tilesize]
        if tileval >= 0:
            tilecoords = resources.tiles_coords[tileval]
            self.bglayer_t.blit(resources.tiles, (coords[0],coords[1]), area=tilecoords)
    
    def RenderMidLayer(self, screen):
        screen.blit(self.bglayer_t, (0,0), area=(self.h_camera.x-self.h_camera.w_view/2, self.h_camera.y-self.h_camera.h_view/2, self.h_camera.w_view, self.h_camera.h_view))
    
    def RenderFGLayer(self, screen):
        screen.blit(self.bglayer_tfg, (0,0), area=(self.h_camera.x-self.h_camera.w_view/2, self.h_camera.y-self.h_camera.h_view/2, self.h_camera.w_view, self.h_camera.h_view))
    
    def InsertObj(self,tileind,objid):
        self.objectlayer[tileind].append(objid)
    
    def RemoveObj(self,tileind,objid):
        if objid in self.objectlayer[tileind]:
            self.objectlayer[tileind].pop(self.objectlayer[tileind].index(objid))
    
    def UpdateObj(self,tileind_old,tileind_new,objid):
        self.RemoveObj(tileind_old,objid)
        self.InsertObj(tileind_new,objid)
    
    def Update(self):
        self.UpdateBlocks()
        self.buttoncontrol.UpdateButtons()
        self.hurtblockcontrol.UpdateHurtBlocks()
        self.platformcontrol.UpdatePlatforms()
        self.doorcontrol.UpdateDoors()
        self.batgroup.Update()
    
    def UpdateBlocks(self):
        for b in self.movingblocks:
            b.UpdateMotion()
            b.UpdateSprite()
    
    def FindMechanisms(self,tile,type):
        tilediffs = [-self.map_size[0]-1,-self.map_size[0],-self.map_size[0]+1, \
            -1,0,1, \
            self.map_size[0]-1,self.map_size[0],self.map_size[0]+1]
        idlist = []
        # Object layer reference:
        #
        # 0: player
        # 100-199: moveable boxes
        # 200-299: buttons
        # 300-399: platforms
        # 400-499: hurtblock
        # 500-599: doors
        # 1000-1999: killer bats
        if type == 'block':
            minid = 100
            maxid = 200
        elif type == 'button':
            minid = 200
            maxid = 300
        elif type == 'platform':
            minid = 300
            maxid = 400
        elif type == 'hurtblock':
            minid = 400
            maxid = 500
        elif type == 'door':
            minid = 500
            maxid = 600
        for tilediff in tilediffs:
            for id in self.objectlayer[tile+tilediff]:
                if id >= minid and id < maxid:
                    idlist.append(id)
        return idlist
    
    """
    def FindBlocks(self,tile):
        tilediffs = [-self.map_size[0]-1,-self.map_size[0],-self.map_size[0]+1, \
            -1,0,1, \
            self.map_size[0]-1,self.map_size[0],self.map_size[0]+1]
        idlist = []
        for tilediff in tilediffs:
            for id in self.objectlayer[tile+tilediff]:
                if id >= 100 and id < 200:
                    idlist.append(id)
        return idlist
    
    def FindPlatforms(self,tile):
        tilediffs = [-self.map_size[0]-1,-self.map_size[0],-self.map_size[0]+1, \
            -1,0,1, \
            self.map_size[0]-1,self.map_size[0],self.map_size[0]+1]
        idlist = []
        for tilediff in tilediffs:
            for id in self.objectlayer[tile+tilediff]:
                if id >= 300 and id < 400:
                    idlist.append(id)
        return idlist
    """
    
    def HandleObjectWallCollision(self,objpos,objsize,objmovevec):
        
        x = objpos[0]
        y = objpos[1]
        vx = objmovevec[0]
        vy = objmovevec[1]
        sx = objsize[0]
        sy = objsize[1]
        ts = self.tilesize
        
        tileocc_ul = self.map_size[0]*((y-(sy/2))/ts)+((x-(sx/2))/ts)
        tileocc_ur = self.map_size[0]*((y-(sy/2))/ts)+((x+(sx/2))/ts)
        tileocc_ll = self.map_size[0]*((y+(sy/2))/ts)+((x-(sx/2))/ts)
        tileocc_lr = self.map_size[0]*((y+(sy/2))/ts)+((x+(sx/2))/ts)
        
        tileocc_u = self.map_size[0]*((y-(sy/2))/ts)+(x/ts)
        tileocc_d = self.map_size[0]*((y+(sy/2))/ts)+(x/ts)
        tileocc_l = self.map_size[0]*(y/ts)+((x-(sx/2))/ts)
        tileocc_r = self.map_size[0]*(y/ts)+((x+(sx/2))/ts)
        
        if vx > 0 and vy == 0: # handle lateral/longitudinal collisions
            if self.occlayer[tileocc_ur] or self.occlayer[tileocc_lr] or self.occlayer[tileocc_r]:
                x = ts*(tileocc_ur % self.map_size[0]) - 1 - sx/2
        elif vx < 0 and vy == 0:
            if self.occlayer[tileocc_ul] or self.occlayer[tileocc_ll] or self.occlayer[tileocc_l]:
                x = ts*(tileocc_ul % self.map_size[0]) + 1 + sx/2 + ts
        elif vx == 0 and vy > 0:
            if self.occlayer[tileocc_lr] or self.occlayer[tileocc_ll] or self.occlayer[tileocc_d]:
                y = ts*(tileocc_ll/self.map_size[0]) - 1 - sy/2
        elif vx == 0 and vy < 0:
            if self.occlayer[tileocc_ul] or self.occlayer[tileocc_ur] or self.occlayer[tileocc_u]:
                y = ts*(tileocc_ul/self.map_size[0]) + 1 + sy/2 + ts
        elif vx > 0 and vy < 0: # handle corner cases
            xref = ts*(tileocc_ur % self.map_size[0]) - 1 - sx/2
            yref = ts*(tileocc_ur/self.map_size[0]) + 1 + sy/2 + ts
            if self.occlayer[tileocc_ul] and self.occlayer[tileocc_lr]:
                x = xref
                y = yref
            elif self.occlayer[tileocc_ul]:
                y = yref
            elif self.occlayer[tileocc_lr]:
                x = xref
            elif self.occlayer[tileocc_ur]:
                if fabs(x-xref) < fabs(y-yref) or self.occlayer[tileocc_ur+self.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx > 0 and vy > 0:
            xref = ts*(tileocc_lr % self.map_size[0]) - 1 - sx/2
            yref = ts*(tileocc_lr/self.map_size[0]) - 1 - sy/2
            if self.occlayer[tileocc_ll] and self.occlayer[tileocc_ur]:
                x = xref
                y = yref
            elif self.occlayer[tileocc_ll]:
                y = yref
            elif self.occlayer[tileocc_ur]:
                x = xref
            elif self.occlayer[tileocc_lr]:
                if fabs(x-xref) < fabs(y-yref) or self.occlayer[tileocc_lr-self.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy > 0:
            xref = ts*(tileocc_ll % self.map_size[0]) + 1 + sx/2 + ts
            yref = ts*(tileocc_ll/self.map_size[0]) - 1 - sy/2
            if self.occlayer[tileocc_ul] and self.occlayer[tileocc_lr]:
                x = xref
                y = yref
            elif self.occlayer[tileocc_lr]:
                y = yref
            elif self.occlayer[tileocc_ul]:
                x = xref
            elif self.occlayer[tileocc_ll]:
                if fabs(x-xref) < fabs(y-yref) or self.occlayer[tileocc_ll-self.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy < 0:
            xref = ts*(tileocc_ul % self.map_size[0]) + 1 + sx/2 + ts
            yref = ts*(tileocc_ul/self.map_size[0]) + 1 + sy/2 + ts
            if self.occlayer[tileocc_ur] and self.occlayer[tileocc_ll]:
                x = xref
                y = yref
            elif self.occlayer[tileocc_ur]:
                y = yref
            elif self.occlayer[tileocc_ll]:
                x = xref
            elif self.occlayer[tileocc_ul]:
                if fabs(x-xref) < fabs(y-yref) or self.occlayer[tileocc_ul+self.map_size[0]]:
                    x = xref
                else:
                    y = yref
        
        return (x,y)
    
    
